@@ -3,8 +3,9 @@
 
 .. versionadded:: 0.5.0
 """
-from ctypes import (CFUNCTYPE, POINTER, c_void_p, c_int, c_size_t, c_double,
-                    c_char_p, c_ubyte, c_bool)
+from ctypes import (CFUNCTYPE, POINTER, c_bool, c_char_p, c_double, c_int,
+                    c_size_t, c_ubyte, c_void_p)
+
 from wand.cdefs.wandtypes import c_ssize_t
 
 __all__ = ('MagickProgressMonitor', 'load')
@@ -132,6 +133,12 @@ def load(lib, IM_VERSION):
             lib.MagickAutoThresholdImage = None
     else:
         lib.MagickAutoThresholdImage = None
+    if IM_VERSION >= 0x711:
+        lib.MagickBilateralBlurImage.argtypes = [c_void_p, c_double, c_double,
+                                                 c_double, c_double]
+        lib.MagickBilateralBlurImage.restype = c_bool
+    else:
+        lib.MagickBilateralBlurImage = None
     lib.MagickBlackThresholdImage.argtypes = [c_void_p, c_void_p]
     lib.MagickBlackThresholdImage.restype = c_bool
     lib.MagickBlueShiftImage.argtypes = [c_void_p, c_double]
@@ -154,6 +161,11 @@ def load(lib, IM_VERSION):
             c_void_p, c_int, c_double, c_double
         ]
         lib.MagickBrightnessContrastImageChannel.restype = c_bool
+    if is_im_7:
+        lib.MagickChannelFxImage.argtypes = [c_void_p, c_char_p]
+        lib.MagickChannelFxImage.restype = c_void_p
+    else:
+        lib.MagickChannelFxImage = None
     if IM_VERSION >= 0x708:
         try:
             lib.MagickCannyEdgeImage.argtypes = [c_void_p, c_double, c_double,
@@ -360,10 +372,16 @@ def load(lib, IM_VERSION):
         lib.MagickFilterImageChannel.restype = c_bool
     lib.MagickFlipImage.argtypes = [c_void_p]
     lib.MagickFlipImage.restype = c_bool
-    lib.MagickFloodfillPaintImage.argtypes = [
-        c_void_p, c_int, c_void_p, c_double, c_void_p, c_ssize_t, c_ssize_t,
-        c_bool
-    ]
+    if is_im_6:
+        lib.MagickFloodfillPaintImage.argtypes = [
+            c_void_p, c_int, c_void_p, c_double, c_void_p,
+            c_ssize_t, c_ssize_t, c_bool
+        ]
+    else:
+        lib.MagickFloodfillPaintImage.argtypes = [
+            c_void_p, c_void_p, c_double, c_void_p,
+            c_ssize_t, c_ssize_t, c_bool
+        ]
     lib.MagickFloodfillPaintImage.restype = c_bool
     lib.MagickFlopImage.argtypes = [c_void_p]
     lib.MagickFlopImage.restype = c_bool
@@ -487,6 +505,14 @@ def load(lib, IM_VERSION):
         lib.MagickGetImageFeatures.restype = c_void_p
     lib.MagickGetImageFilename.argtypes = [c_void_p]
     lib.MagickGetImageFilename.restype = c_void_p
+    if IM_VERSION >= 0x710:
+        try:
+            lib.MagickGetImageFilter.argtypes = [c_void_p]
+            lib.MagickGetImageFilter.restype = c_int
+        except AttributeError:
+            lib.MagickGetImageFilter = None
+    else:
+        lib.MagickGetImageFilter = None
     lib.MagickGetImageFormat.argtypes = [c_void_p]
     lib.MagickGetImageFormat.restype = c_void_p
     lib.MagickGetImageFuzz.argtypes = [c_void_p]
@@ -619,6 +645,11 @@ def load(lib, IM_VERSION):
         lib.MagickHoughLineImage = None
     lib.MagickIdentifyImage.argtypes = [c_void_p]
     lib.MagickIdentifyImage.restype = c_void_p
+    if IM_VERSION >= 0x700:
+        lib.MagickIdentifyImageType.argtypes = [c_void_p]
+        lib.MagickIdentifyImageType.restype = c_int
+    else:
+        lib.MagickIdentifyImageType = None
     if is_im_6:
         lib.MagickImplodeImage.argtypes = [c_void_p, c_double]
     else:
@@ -984,6 +1015,14 @@ def load(lib, IM_VERSION):
     lib.MagickSetImageExtent.restype = c_bool
     lib.MagickSetImageFilename.argtypes = [c_void_p, c_char_p]
     lib.MagickSetImageFilename.restype = c_bool
+    if IM_VERSION >= 0x710:
+        try:
+            lib.MagickSetImageFilter.argtypes = [c_void_p, c_int]
+            lib.MagickSetImageFilter.restype = c_bool
+        except AttributeError:
+            lib.MagickSetImageFilter = None
+    else:
+        lib.MagickSetImageFilter = None
     lib.MagickSetImageFormat.argtypes = [c_void_p, c_char_p]
     lib.MagickSetImageFormat.restype = c_bool
     lib.MagickSetImageFuzz.argtypes = [c_void_p, c_double]
