@@ -12,7 +12,7 @@ or your default image application (:program:`Windows Photo Viewer` usually)
 if :program:`imdisplay.exe` is unavailable.
 
 You can use it from CLI also.  Execute :mod:`wand.display` module through
-:option:`python -m` option:
+:option:`python -m <-m>` option:
 
 .. sourcecode:: console
 
@@ -27,9 +27,9 @@ import platform
 import sys
 import tempfile
 
-from .image import Image
 from .api import library
-from .exceptions import BlobError, DelegateError
+from .exceptions import BlobError, CoderError, DelegateError
+from .image import Image
 
 __all__ = 'display',
 
@@ -51,13 +51,15 @@ def display(image, server_name=':0'):
     if system == 'Windows':
         try:
             image.save(filename='win:.')
-        except DelegateError:
+        except (BlobError, CoderError, DelegateError):
             pass
         else:
             return
     if system in ('Windows', 'Darwin'):
-        ext = '.' + image.format.lower()
-        path = tempfile.mktemp(suffix=ext)
+        ext = image.format.lower()
+        if ext in ('miff', 'xc'):
+            ext = 'png'
+        path = tempfile.mktemp(suffix='.' + ext)
         image.save(filename=path)
         os.system(('start ' if system == 'Windows' else 'open ') + path)
     else:
@@ -68,12 +70,11 @@ def display(image, server_name=':0'):
 
 if __name__ == '__main__':
     if len(sys.argv) < 2:
-        print>>sys.stderr, 'usage: python -m wand.display FILE'
+        print('usage: python -m wand.display FILE', file=sys.stderr)
         raise SystemExit
     path = sys.argv[1]
     try:
         with Image(filename=path) as image:
             display(image)
     except BlobError:
-        print>>sys.stderr, 'cannot read the file', path
-
+        print('cannot read the file', path, file=sys.stderr)
