@@ -9,15 +9,17 @@ The module provides some vector drawing functions.
 import collections
 import ctypes
 import numbers
+from collections import abc
 
 from . import assertions
-from .api import AffineMatrix, PointInfo, library
+from .api import library
+from .cdefs.structures import AffineMatrix, PointInfo
 from .color import Color
-from .compat import abc, binary, string_type, text, text_type, xrange
+from .compat import binary, text
 from .exceptions import WandLibraryVersionError
-from .image import BaseImage, COMPOSITE_OPERATORS
-from .sequence import SingleImage
+from .image import COMPOSITE_OPERATORS, BaseImage
 from .resource import Resource
+from .sequence import SingleImage
 
 __all__ = ('CLIP_PATH_UNITS', 'FILL_RULE_TYPES', 'FONT_METRICS_ATTRIBUTES',
            'GRAVITY_TYPES', 'LINE_CAP_TYPES', 'LINE_JOIN_TYPES',
@@ -90,9 +92,6 @@ FONT_METRICS_ATTRIBUTES = ('character_width', 'character_height', 'ascender',
                            'maximum_horizontal_advance', 'x1', 'y1', 'x2',
                            'y2', 'x', 'y')
 
-#: The tuple subtype which consists of font metrics data.
-FontMetrics = collections.namedtuple('FontMetrics', FONT_METRICS_ATTRIBUTES)
-
 #: (:class:`collections.abc.Sequence`) The list of stretch types for fonts
 #:
 #: - ``'undefined;``
@@ -200,7 +199,7 @@ class Drawing(Resource):
 
     @border_color.setter
     def border_color(self, color):
-        if isinstance(color, string_type):
+        if isinstance(color, str):
             color = Color(color)
         assertions.assert_color(border_color=color)
         with color:
@@ -208,7 +207,7 @@ class Drawing(Resource):
 
     @property
     def clip_path(self):
-        """(:class:`basestring`) The current clip path. It also can be set.
+        """(:class:`str`) The current clip path. It also can be set.
 
         .. versionadded:: 0.4.0
 
@@ -231,7 +230,7 @@ class Drawing(Resource):
 
     @property
     def clip_rule(self):
-        """(:class:`basestring`) The current clip rule. It also can be set.
+        """(:class:`str`) The current clip rule. It also can be set.
         It's a string value from :const:`FILL_RULE_TYPES` list.
 
         .. versionadded:: 0.4.0
@@ -249,7 +248,7 @@ class Drawing(Resource):
 
     @property
     def clip_units(self):
-        """(:class:`basestring`) The current clip units. It also can be set.
+        """(:class:`str`) The current clip units. It also can be set.
         It's a string value from :const:`CLIP_PATH_UNITS` list.
 
         .. versionadded:: 0.4.0
@@ -279,7 +278,7 @@ class Drawing(Resource):
 
     @fill_color.setter
     def fill_color(self, color):
-        if isinstance(color, string_type):
+        if isinstance(color, str):
             color = Color(color)
         assertions.assert_color(fill_color=color)
         with color:
@@ -301,7 +300,7 @@ class Drawing(Resource):
 
     @property
     def fill_rule(self):
-        """(:class:`basestring`) The current fill rule. It can also be set.
+        """(:class:`str`) The current fill rule. It can also be set.
         It's a string value from :const:`FILL_RULE_TYPES` list.
 
         .. versionadded:: 0.4.0
@@ -309,7 +308,7 @@ class Drawing(Resource):
         fill_rule_index = library.DrawGetFillRule(self.resource)
         if fill_rule_index not in FILL_RULE_TYPES:
             self.raise_exception()
-        return text(FILL_RULE_TYPES[fill_rule_index])
+        return FILL_RULE_TYPES[fill_rule_index]
 
     @fill_rule.setter
     def fill_rule(self, fill_rule):
@@ -321,7 +320,7 @@ class Drawing(Resource):
 
     @property
     def font(self):
-        """(:class:`basestring`) The current font name.  It also can be set.
+        """(:class:`str`) The current font name.  It also can be set.
 
         .. versionchanged: 0.4.1
            Safely release allocated memory with
@@ -342,7 +341,7 @@ class Drawing(Resource):
 
     @property
     def font_family(self):
-        """(:class:`basestring`) The current font family. It also can be set.
+        """(:class:`str`) The current font family. It also can be set.
 
         .. versionadded:: 0.4.0
 
@@ -365,8 +364,8 @@ class Drawing(Resource):
 
     @property
     def font_resolution(self):
-        """(:class:`~collections.abc.Sequence`) The current font resolution. It also
-        can be set.
+        """(:class:`~collections.abc.Sequence`) The current font resolution.
+        It also can be set.
 
         .. versionadded:: 0.4.0
         """
@@ -390,19 +389,19 @@ class Drawing(Resource):
     def font_size(self, size):
         assertions.assert_real(font_size=size)
         if size < 0.0:
-            raise ValueError('cannot be less then 0.0, but got ' + repr(size))
+            raise ValueError('cannot be less than 0.0, but got ' + repr(size))
         library.DrawSetFontSize(self.resource, size)
 
     @property
     def font_stretch(self):
-        """(:class:`basestring`) The current font stretch variation.
+        """(:class:`str`) The current font stretch variation.
         It also can be set, but will only apply if the font-family or encoder
         supports the stretch type.
 
         .. versionadded:: 0.4.0
         """
         stretch_index = library.DrawGetFontStretch(self.resource)
-        return text(STRETCH_TYPES[stretch_index])
+        return STRETCH_TYPES[stretch_index]
 
     @font_stretch.setter
     def font_stretch(self, stretch):
@@ -414,14 +413,14 @@ class Drawing(Resource):
 
     @property
     def font_style(self):
-        """(:class:`basestring`) The current font style.
+        """(:class:`str`) The current font style.
         It also can be set, but will only apply if the font-family
         supports the style.
 
         .. versionadded:: 0.4.0
         """
         style_index = library.DrawGetFontStyle(self.resource)
-        return text(STYLE_TYPES[style_index])
+        return STYLE_TYPES[style_index]
 
     @font_style.setter
     def font_style(self, style):
@@ -447,7 +446,7 @@ class Drawing(Resource):
 
     @property
     def gravity(self):
-        """(:class:`basestring`) The text placement gravity used when
+        """(:class:`str`) The text placement gravity used when
         annotating with text.  It's a string from :const:`GRAVITY_TYPES`
         list.  It also can be set.
 
@@ -455,7 +454,7 @@ class Drawing(Resource):
         gravity_index = library.DrawGetGravity(self.resource)
         if not gravity_index:
             self.raise_exception()
-        return text(GRAVITY_TYPES[gravity_index])
+        return GRAVITY_TYPES[gravity_index]
 
     @gravity.setter
     def gravity(self, value):
@@ -517,7 +516,7 @@ class Drawing(Resource):
 
     @stroke_color.setter
     def stroke_color(self, color):
-        if isinstance(color, string_type):
+        if isinstance(color, str):
             color = Color(color)
         assertions.assert_color(stroke_color=color)
         with color:
@@ -543,7 +542,7 @@ class Drawing(Resource):
         dash_array = []
         if dash_array_p:
             dash_array = [float(dash_array_p[i])
-                          for i in xrange(number_elements.value)]
+                          for i in range(number_elements.value)]
             dash_array_p = library.MagickRelinquishMemory(dash_array_p)
         return dash_array
 
@@ -570,14 +569,14 @@ class Drawing(Resource):
 
     @property
     def stroke_line_cap(self):
-        """(:class:`basestring`) The stroke line cap. It also can be set.
+        """(:class:`str`) The stroke line cap. It also can be set.
 
         .. versionadded:: 0.4.0
         """
         line_cap_index = library.DrawGetStrokeLineCap(self.resource)
         if line_cap_index not in LINE_CAP_TYPES:
             self.raise_exception()
-        return text(LINE_CAP_TYPES[line_cap_index])
+        return LINE_CAP_TYPES[line_cap_index]
 
     @stroke_line_cap.setter
     def stroke_line_cap(self, line_cap):
@@ -589,14 +588,14 @@ class Drawing(Resource):
 
     @property
     def stroke_line_join(self):
-        """(:class:`basestring`) The stroke line join. It also can be set.
+        """(:class:`str`) The stroke line join. It also can be set.
 
         .. versionadded:: 0.4.0
         """
         line_join_index = library.DrawGetStrokeLineJoin(self.resource)
         if line_join_index not in LINE_JOIN_TYPES:
             self.raise_exception()
-        return text(LINE_JOIN_TYPES[line_join_index])
+        return LINE_JOIN_TYPES[line_join_index]
 
     @stroke_line_join.setter
     def stroke_line_join(self, line_join):
@@ -647,12 +646,12 @@ class Drawing(Resource):
     def stroke_width(self, width):
         assertions.assert_real(stroke_width=width)
         if width < 0.0:
-            raise ValueError('cannot be less then 0.0, but got ' + repr(width))
+            raise ValueError('cannot be less than 0.0, but got ' + repr(width))
         library.DrawSetStrokeWidth(self.resource, width)
 
     @property
     def text_alignment(self):
-        """(:class:`basestring`) The current text alignment setting.
+        """(:class:`str`) The current text alignment setting.
         It's a string value from :const:`TEXT_ALIGN_TYPES` list.
         It also can be set.
 
@@ -660,7 +659,7 @@ class Drawing(Resource):
         text_alignment_index = library.DrawGetTextAlignment(self.resource)
         if not text_alignment_index:  # pragma: no cover
             self.raise_exception()
-        return text(TEXT_ALIGN_TYPES[text_alignment_index])
+        return TEXT_ALIGN_TYPES[text_alignment_index]
 
     @text_alignment.setter
     def text_alignment(self, align):
@@ -687,14 +686,14 @@ class Drawing(Resource):
 
     @property
     def text_decoration(self):
-        """(:class:`basestring`) The text decoration setting, a string
+        """(:class:`str`) The text decoration setting, a string
         from :const:`TEXT_DECORATION_TYPES` list.  It also can be set.
 
         """
         text_decoration_index = library.DrawGetTextDecoration(self.resource)
         if not text_decoration_index:  # pragma: no cover
             self.raise_exception()
-        return text(TEXT_DECORATION_TYPES[text_decoration_index])
+        return TEXT_DECORATION_TYPES[text_decoration_index]
 
     @text_decoration.setter
     def text_decoration(self, decoration):
@@ -706,7 +705,7 @@ class Drawing(Resource):
 
     @property
     def text_direction(self):
-        """(:class:`basestring`) The text direction setting. a string
+        """(:class:`str`) The text direction setting. a string
         from :const:`TEXT_DIRECTION_TYPES` list. It also can be set."""
         if library.DrawGetTextDirection is None:  # pragma: no cover
             raise WandLibraryVersionError(
@@ -716,7 +715,7 @@ class Drawing(Resource):
         text_direction_index = library.DrawGetTextDirection(self.resource)
         if not text_direction_index:  # pragma: no cover
             self.raise_exception()
-        return text(TEXT_DIRECTION_TYPES[text_direction_index])
+        return TEXT_DIRECTION_TYPES[text_direction_index]
 
     @text_direction.setter
     def text_direction(self, direction):
@@ -733,7 +732,7 @@ class Drawing(Resource):
 
     @property
     def text_encoding(self):
-        """(:class:`basestring`) The internally used text encoding setting.
+        """(:class:`str`) The internally used text encoding setting.
         Although it also can be set, but it's not encouraged.
 
         .. versionchanged: 0.4.1
@@ -820,7 +819,7 @@ class Drawing(Resource):
 
     @text_under_color.setter
     def text_under_color(self, color):
-        if isinstance(color, string_type):
+        if isinstance(color, str):
             color = Color(color)
         assertions.assert_color(text_under_color=color)
         with color:
@@ -828,7 +827,7 @@ class Drawing(Resource):
 
     @property
     def vector_graphics(self):
-        """(:class:`basestring`) The XML text of the Vector Graphics.
+        """(:class:`str`) The XML text of the Vector Graphics.
         It also can be set.  The drawing-wand XML is experimental,
         and subject to change.
 
@@ -853,7 +852,7 @@ class Drawing(Resource):
     @vector_graphics.setter
     def vector_graphics(self, vector_graphics):
         if vector_graphics is not None and not isinstance(vector_graphics,
-                                                          string_type):
+                                                          str):
             raise TypeError('expected a string, not ' + repr(vector_graphics))
         elif vector_graphics is None:
             # Reset all vector graphic properties on drawing wand.
@@ -895,8 +894,8 @@ class Drawing(Resource):
         library.DrawAffine(self.resource, ctypes.byref(amx))
 
     def alpha(self, x=None, y=None, paint_method='undefined'):
-        """Paints on the image's opacity channel in order to set effected pixels
-        to transparent.
+        """Paints on the image's opacity channel in order to set effected
+        pixels to transparent.
 
          To influence the opacity of pixels. The available methods are:
 
@@ -1043,7 +1042,7 @@ class Drawing(Resource):
         """Adds a comment to the vector stream.
 
         :param message: the comment to set.
-        :type message: :class:`basestring`
+        :type message: :class:`str`
 
         .. versionadded:: 0.4.0
         """
@@ -1145,15 +1144,25 @@ class Drawing(Resource):
                             rotation_start, rotation_end)
 
     def get_font_metrics(self, image, text, multiline=False):
-        """Queries font metrics from the given ``text``.
+        """Queries font metrics by cloning the ``image``, and rendering
+        the ``text`` with the font properties defined on :class:`Drawing`.
+
+        This is useful for determining the size of the rendered text. Set
+        ``multiline`` to ``True`` if the given ``text`` contains line-breaks.
+
+        The return value is an instance of :class:`FontMetrics` which
+        has the attributes of the text's rendered size, max horizontal advance,
+        and distance of ascent & descent from the baseline.
 
         :param image: the image to be drawn
         :type image: :class:`~wand.image.BaseImage`
         :param text: the text string for get font metrics.
-        :type text: :class:`basestring`
+        :type text: :class:`str`
         :param multiline: text is multiline or not
         :type multiline: `boolean`
+        :returns: :class:`FontMetrics`
 
+        .. versionadded:: 0.3.0
         """
         if not isinstance(image, BaseImage):
             raise TypeError('image must be a wand.image.BaseImage instance,'
@@ -1163,7 +1172,7 @@ class Drawing(Resource):
             font_metrics_f = library.MagickQueryMultilineFontMetrics
         else:
             font_metrics_f = library.MagickQueryFontMetrics
-        if isinstance(text, text_type):
+        if isinstance(text, str):
             if self.text_encoding:
                 text = text.encode(self.text_encoding)
             else:
@@ -1176,7 +1185,7 @@ class Drawing(Resource):
             image.raise_exception()
             # Generate a generic error if ImageMagick couldn't emit one.
             raise ValueError('Unable to render text with current font.')
-        args = [result[i] for i in xrange(13)]
+        args = [result[i] for i in range(13)]
         result = library.MagickRelinquishMemory(result)
         return FontMetrics(*args)
 
@@ -1198,8 +1207,8 @@ class Drawing(Resource):
                          int(end_x), int(end_y))
 
     def matte(self, x=0.0, y=0.0, paint_method='undefined'):
-        """Paints on the image's opacity channel in order to set effected pixels
-        to transparent.
+        """Paints on the image's opacity channel in order to set effected
+        pixels to transparent.
 
          To influence the opacity of pixels. The available methods are:
 
@@ -1624,7 +1633,7 @@ class Drawing(Resource):
         the previous style attributes. Use :class:`Drawing.pop` to return
         to restore previous style attributes.
 
-        This is useful for drawing shapes with diffrent styles
+        This is useful for drawing shapes with different styles
         without repeatedly setting the similar
         :meth:`fill_color <wand.drawing.Drawing.fill_color>` &
         :meth:`stroke_color <wand.drawing.Drawing.stroke_color>` properties.
@@ -1682,7 +1691,7 @@ class Drawing(Resource):
         command.
 
         :param clip_mask_id: string identifier to associate with the clip path.
-        :type clip_mask_id: :class:`basestring`
+        :type clip_mask_id: :class:`str`
 
         .. versionadded:: 0.4.0
 
@@ -1690,9 +1699,10 @@ class Drawing(Resource):
         library.DrawPushClipPath(self.resource, binary(clip_mask_id))
 
     def push_defs(self):
-        """Indicates that commands up to a terminating :class:`Drawing.pop_defs`
-        command create named elements (e.g. clip-paths, textures, etc.) which
-        may safely be processed earlier for the sake of efficiency.
+        """Indicates that commands up to a terminating
+        :class:`Drawing.pop_defs` command create named elements (e.g.
+        clip-paths, textures, etc.) which may safely be processed earlier for
+        the sake of efficiency.
 
         .. versionadded:: 0.4.0
 
@@ -1708,7 +1718,7 @@ class Drawing(Resource):
         Named patterns may be used as stroke or brush definitions.
 
         :param pattern_id: a unique identifier for the pattern.
-        :type pattern_id: :class:`basestring`
+        :type pattern_id: :class:`str`
         :param left: x ordinate of top left corner.
         :type left: :class:`numbers.Real`
         :param top: y ordinate of top left corner.
@@ -1874,13 +1884,13 @@ class Drawing(Resource):
         library.DrawScale(self.resource, x, y)
 
     def set_fill_pattern_url(self, url):
-        """Sets the URL to use as a fill pattern for filling objects. Only local
-        URLs ("#identifier") are supported at this time. These local URLs are
-        normally created by defining a named fill pattern with
+        """Sets the URL to use as a fill pattern for filling objects. Only
+        local URLs ("#identifier") are supported at this time. These local URLs
+        are normally created by defining a named fill pattern with
         Drawing.push_pattern & Drawing.pop_pattern.
 
         :param url: URL to use to obtain fill pattern.
-        :type url: :class:`basestring`
+        :type url: :class:`str`
 
         .. versionadded:: 0.4.0
 
@@ -1899,7 +1909,7 @@ class Drawing(Resource):
         Drawing.push_pattern & Drawing.pop_pattern.
 
         :param url: URL to use to obtain stroke pattern.
-        :type url: :class:`basestring`
+        :type url: :class:`str`
 
         .. versionadded:: 0.4.0
 
@@ -1938,14 +1948,14 @@ class Drawing(Resource):
         :param y: the baseline where to start writing text
         :type y: :class:`numbers.Integral`
         :param body: the body string to write
-        :type body: :class:`basestring`
+        :type body: :class:`str`
 
         """
         assertions.assert_unsigned_integer(x=x, y=y)
         assertions.assert_string(body=body)
         if not body:
             raise ValueError('body string cannot be empty')
-        if isinstance(body, text_type):
+        if isinstance(body, str):
             # According to ImageMagick C API docs, we can use only UTF-8
             # at this time, so we do hardcoding here.
             # http://imagemagick.org/api/drawing-wand.php#DrawSetTextEncoding
@@ -1999,6 +2009,100 @@ class Drawing(Resource):
         return self.draw(image)
 
 
+class FontMetrics(collections.namedtuple('FontMetrics',
+                                         FONT_METRICS_ATTRIBUTES)):
+    """
+    The tuple subtype which consists of font metrics data.
+
+    Pixel values can be converted to points by the following:
+
+    .. math::
+
+        points = \\frac{pixels \\ast 72}{resolution}
+
+    .. py:attribute:: character_width
+
+        (:class:`numbers.Real`) The horizontal value of the ``x`` in pixels.
+
+    .. py:attribute:: character_height
+
+        (:class:`numbers.Real`) The vertical value of the ``x`` in pixels.
+
+    .. py:attribute:: ascender
+
+        (:class:`numbers.Real`) The largest distance from the text baseline
+        to the highest point above the x-height. This value should always be
+        positive.
+
+    .. py:attribute:: descender
+
+        (:class:`numbers.Real`) The farthest distance from the text baseline
+        to the lowest point below the x-height. This value should always be
+        negative.
+
+    .. py:attribute:: text_width
+
+        (:class:`numbers.Real`) The full width of the text.
+
+    .. py:attribute:: text_height
+
+        (:class:`numbers.Real`) The full height of the text.
+
+    .. py:attribute:: maximum_horizontal_advance
+
+        (:class:`numbers.Real`) The largest distance from the start of one
+        character to the start of the following character.
+
+    .. py:attribute:: x1
+
+        (:class:`numbers.Real`) The horizontal value of the bounding box's
+        starting corner.
+
+    .. py:attribute:: y1
+
+        (:class:`numbers.Real`) The vertical value of the bounding box's
+        starting corner.
+
+    .. py:attribute:: x2
+
+        (:class:`numbers.Real`) The horizontal value of the bounding box's
+        ending corner.
+
+    .. py:attribute:: y2
+
+        (:class:`numbers.Real`) The vertical value of the bounding box's
+        ending corner.
+
+    .. py:attribute:: x
+
+        (:class:`numbers.Real`) The horizontal value of the origin.
+
+    .. py:attribute:: y
+
+        (:class:`numbers.Real`) The vertical value of the origin.
+
+    .. note::
+
+        When using :meth:`~wand.drawing.Drawing.get_font_metrics`, the returned
+        attributes ``x1``, ``y1``, ``x2``, ``y2``, ``x`` and ``y`` can be
+        ignored as they are used for internal FreeType rendering, and have no
+        meaningful context.
+
+    .. versionadded:: 0.3.0
+
+    .. versionchanged:: 0.6.8
+       Created sub-class of the namedtuple to improve auto-documentation.
+    """
+    def size(self):
+        """Short-cut method for the width & height of the rendered text.
+
+        :returns: (width, height)
+
+        .. versionadded:: 0.6.8
+        """
+        return self.text_width, self.text_height
+
+
 def _list_to_point_info(points):
     """
     Helper method to convert a list of tuples to ``const * PointInfo``
@@ -2019,7 +2123,7 @@ def _list_to_point_info(points):
     point_info_size = point_length * tuple_size
     # Allocate sequence of memory
     point_info = (ctypes.c_double * point_info_size)()
-    for double_index in xrange(point_info_size):
+    for double_index in range(point_info_size):
         tuple_index = double_index // tuple_size
         tuple_offset = double_index % tuple_size
         point_info[double_index] = ctypes.c_double(
